
@font-face {
  font-family: 'Ubuntu-RI';
  src: url('/static/fonts/ubuntu-font-family-0.83/Ubuntu-RI.ttf') format('truetype');
}

@font-face {
  font-family: 'Ubuntu-LI';
  src: url('/static/fonts/ubuntu-font-family-0.83/Ubuntu-LI.ttf') format('truetype');
}

@font-face {
  font-family: 'Ubuntu-MI';
  src: url('/static/fonts/ubuntu-font-family-0.83/Ubuntu-MI.ttf') format('truetype');
}

@font-face {
  font-family: 'Ubuntu-M';
  src: url('/static/fonts/ubuntu-font-family-0.83/Ubuntu-M.ttf') format('truetype');
}

@font-face {
  font-family: 'Ubuntu-Th';
  src: url('/static/fonts/ubuntu-font-family-0.83/Ubuntu-Th.ttf') format('truetype');
}

@font-face {
  font-family: 'UbuntuMono-BI';
  src: url('/static/fonts/ubuntu-font-family-0.83/UbuntuMono-BI.ttf') format('truetype');
}

@font-face {
  font-family: 'UbuntuMono-RI';
  src: url('/static/fonts/ubuntu-font-family-0.83/UbuntuMono-RI.ttf') format('truetype');
}

@font-face {
  font-family: 'Ubuntu-C';
  src: url('/static/fonts/ubuntu-font-family-0.83/Ubuntu-C.ttf') format('truetype');
}

@font-face {
  font-family: 'Ubuntu-R';
  src: url('/static/fonts/ubuntu-font-family-0.83/Ubuntu-R.ttf') format('truetype');
}

@font-face {
  font-family: 'Ubuntu-BI';
  src: url('/static/fonts/ubuntu-font-family-0.83/Ubuntu-BI.ttf') format('truetype');
}

@font-face {
  font-family: 'UbuntuMono-R';
  src: url('/static/fonts/ubuntu-font-family-0.83/UbuntuMono-R.ttf') format('truetype');
}

@font-face {
  font-family: 'Ubuntu-L';
  src: url('/static/fonts/ubuntu-font-family-0.83/Ubuntu-L.ttf') format('truetype');
}

@font-face {
  font-family: 'Ubuntu-B';
  src: url('/static/fonts/ubuntu-font-family-0.83/Ubuntu-B.ttf') format('truetype');
}

@font-face {
  font-family: 'UbuntuMono-B';
  src: url('/static/fonts/ubuntu-font-family-0.83/UbuntuMono-B.ttf') format('truetype');
}
